@@ -33,14 +33,14 @@
 # MODIFICATIONS.
 from __future__ import print_function
 import fileinput
-import sys
+# import sys
 import argparse
 # import csv
 import os
 import logging
 import pprint as pp
 import subprocess
-import xml.etree.ElementTree as ET
+import xml.etree.ElementTree as ElementTree
 # import json
 import wget
 import urllib
@@ -50,14 +50,17 @@ import shutil
 import gzip
 # from json import JSONDecoder, JSONDecodeError
 import json
-from tqdm import tqdm
+# from tqdm import tqdm
 # from collections import defaultdict
+from rich.logging import RichHandler
+from rich.progress import track
 from signal import signal, SIGPIPE, SIGINT, SIG_DFL
 from .__version__ import __version__
 signal(SIGPIPE, SIG_DFL)
 signal(SIGINT, SIG_DFL)
 
 NOT_WHITESPACE = re.compile(r'[^\s]')
+OUTPUTS = ('fna', 'ffn', 'gff', 'gbk', 'faa')
 
 
 class DownloadEdirect(argparse.Action):
@@ -65,6 +68,8 @@ class DownloadEdirect(argparse.Action):
         super().__init__(nargs=nargs, **kw)
 
     def __call__(self, parser, namespace, values, option_string=None):
+        init_logger(True)
+        logger = logging.getLogger(__name__)
         if values:
             prefix = values
         else:
@@ -82,21 +87,28 @@ class DownloadEdirect(argparse.Action):
         os.chdir(dlpath)
         out_file = os.path.join(dlpath, 'edirect.tar.gz')
         url = 'ftp://ftp.ncbi.nlm.nih.gov/entrez/entrezdirect/edirect.tar.gz'
-        sys.stderr.write('Downloading edirect.tar.gz from NCBI.\n')
-        if not os.path.exists(out_file) or os.path.getsize(out_file) == 0:
-            with urllib.request.urlopen(url) as response, \
-                    open(out_file, 'wb') as tarfh:
-                shutil.copyfileobj(response, tarfh)
-        sys.stderr.write(f'Installing edirect in {prefix}. Running setup.sh\n')
+        dl_file(url, out_file)
+        logger.info(f'Installing edirect in {prefix}. Running setup.sh.')
         shutil.unpack_archive(out_file, dlpath, 'gztar')
         os.remove(out_file)
         subprocess.run([os.path.join(prefix, 'setup.sh')])
-        sys.stderr.write('\nAdd given directory to $PATH to continue.\n')
-        sys.stderr.write('This script will find ~/edirect by default.\n')
-        sys.stderr.write('Alternatively, set $EDIRECT environment variable.\n')
+        logger.info('Add given directory {} to [green]$PATH[/green] to '
+                    'continue.'.format(prefix), extra={'markup': True})
+        logger.info('This script will find ~/edirect by default.')
+        logger.info('Alternatively, set [green]$EDIRECT[/green] environment '
+                    'variable.', extra={'markup': True})
         parser.exit()
 
 
+def dl_file(url, out_file):
+    logger = logging.getLogger(__name__)
+    logger.debug('Downloading file {} to {}'.format(url, out_file))
+    if not os.path.exists(out_file) or os.path.getsize(out_file) == 0:
+        with urllib.request.urlopen(url) as response, \
+                open(out_file, 'wb') as tarfh:
+            shutil.copyfileobj(response, tarfh)
+
+
 def extant_file(x):
     """
     'Type' for argparse - checks that file exists but does not open.
@@ -110,30 +122,36 @@ def edirect_dir(x):
     """
     'Type' for argparse - checks that edirect path is present.
     """
+    init_logger(1)
+    logger = logging.getLogger(__name__)
     if not os.path.exists(os.path.join(x, 'edirect.pl')):
-        msg = '\n#### - ! {} does NOT appear to be a valid edirect path' \
-              ' ! - ####\n'
-        sys.stderr.write(msg.format(x) + '\n')
-        msg = 'Please specify a valid edirect path in the script and try '\
-              'again.'
-        raise argparse.ArgumentTypeError(msg)
+        msg = ('Given path [bold magenta]{}[/bold magenta] does '
+               '[bold red]NOT[/bold red] appear to be a valid edirect path.')
+        logger.error(msg.format(x), extra={'markup': True})
+        msg = ('Please specify a valid edirect path with '
+               '[bold magenta]--edirect[/bold magenta] and try again.')
+        logger.error(msg, extra={'markup': True})
+        exit()
     return x
 
 
-def init_logger(debug):
+def init_logger(debug, logfile=''):
     logger = logging.getLogger()
-    ch = logging.StreamHandler()
+    # ch = logging.StreamHandler()
+    ch = RichHandler(rich_tracebacks=True)
     logger.setLevel(logging.DEBUG)
     if debug:
         ch.setLevel(logging.DEBUG)
     else:
         ch.setLevel(logging.INFO)
-    fh = logging.FileHandler('get_assemblies.log', 'a')
     formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
-    ch.setFormatter(formatter)
-    fh.setFormatter(formatter)
+    # ch.setFormatter(formatter)
     logger.addHandler(ch)
-    logger.addHandler(fh)
+
+    if logfile:
+        fh = logging.FileHandler(logfile, 'a')
+        fh.setFormatter(formatter)
+        logger.addHandler(fh)
 
 
 def run_argparse():
@@ -241,12 +259,12 @@ def run_argparse():
         )
 
     args = parser.parse_args()
-    init_logger(args.debug)
+    init_logger(args.debug, logfile='get_assemblies.log')
     if not args.intype:
         parser.print_help()
         exit()
-    if args.debug:
-        pp.pprint(args)
+    # if args.debug:
+    #     pp.pprint(args)
     return args
 
 
@@ -278,7 +296,6 @@ def eutil_critical(name):
 
 def validate_inputs(args):
     logger = logging.getLogger(__name__)
-    OUTPUTS = ('fna', 'ffn', 'gff', 'gbk', 'faa')
 
     if 'genomes' in args.function and 'metadata' not in args.function:
         if args.intype != 'json_file':
@@ -328,6 +345,7 @@ def validate_inputs(args):
         else:
             eutil_critical(util)
 
+    # logger.debug(pp.pformat(args))
     logger.debug(pp.pformat(args))
     logger.debug(pp.pformat(exes))
     return(args, exes)
@@ -340,23 +358,19 @@ def search_query(esearch, efilter, qtype, query):
     command = []
     if qtype == 'text':
         query = f'{query}[ORGN]'
-        command = [esearch,
-                   '-db', 'assembly',
-                   '-query', query]
     elif qtype == 'ID':
         query = f'txid{query}[ORGN]'
-        command = [esearch,
-                   '-db', 'assembly',
-                   '-query', query]
 
-    filt_cmd = [efilter, '-query', 'latest']
+    query = query + ' AND latest [PROP] NOT '\
+        'partial-genome-representation [PROP]'
 
-    logger.debug('Running command:\n' + ' '.join(command + ['|'] + filt_cmd))
+    command = [esearch, '-db', 'assembly', '-query', query]
+
+    # logger.debug('Running command:\n' + ' '.join(command))
+    logger.debug('Running command:\n' + ' '.join(command))
     search_res = subprocess.run(command, stdout=subprocess.PIPE, text=True)
-    filt_res = subprocess.run(filt_cmd, input=search_res.stdout, text=True,
-                              stdout=subprocess.PIPE)
 
-    return(filt_res.stdout)
+    return(search_res.stdout)
 
 
 def get_assem_links(epost, infile, qtype):
@@ -398,7 +412,7 @@ def convert_nuc_to_assem(elink, infile):
 
 def check_count(assem_links, query='stdin'):
     logger = logging.getLogger(__name__)
-    entrez_direct = ET.fromstring(assem_links)
+    entrez_direct = ElementTree.fromstring(assem_links)
     try:
         count = entrez_direct.find('Count').text
     except AttributeError:
@@ -419,7 +433,7 @@ def check_count(assem_links, query='stdin'):
             f'Found {count} genome(s) to download.'
         )
         logger.info(
-            f'Expect {int(count)*5}MB to {int(count)*7}MB of data pending '
+            f'Expect {int(count)*5} MB to {int(count)*7} MB of data pending '
             'the chosen file types for download.'
         )
     # return(count)
@@ -454,14 +468,15 @@ def fetch_docsums(efetch, assem_links):
                 length = length/60
         logger.info('With {} chunks (500 ids per), this will take around '
                     '{} {}.'.format(nchunk, int(length), units))
-    for chunk in tqdm(chunks(uid_list, 500), 'chunk', nchunk):
+    for chunk in track(chunks(uid_list, 500), 'chunk', nchunk):
         command = [f'{efetch}',
                    '-format', 'docsum',
                    '-mode', 'json',
                    '-db', 'assembly',
                    '-id', ','.join(chunk)]
         # command += ','.join(chunk)
-        logger.debug('Running this command:\n' + ' '.join(command))
+        # logger.debug('Running this command:\n' + ' '.join(command))
+        logger.debug('Running this command:' + ' '.join(command))
         jsondata = []
 
         for j in range(5):
@@ -631,7 +646,7 @@ def extract_metadata(force, metadata_append, outformat, typestrain, annotation,
         metadata.append(json_keys + special_keys + ['prefix'])
 
     # for d in decode_stacked_json(docsums):
-    for d in tqdm(docsums, 'docsums', len(docsums)):
+    for d in track(docsums, 'docsums', len(docsums)):
         try:
             json_data = d['result']
         except KeyError:
@@ -883,7 +898,7 @@ def download_genomes(o, dl_mapping):
 
     nacc = len(dl_mapping)
     logger.info('Downloading {} file(s).'.format(nacc * len(o)))
-    for acc in tqdm(dl_mapping, 'download'):
+    for acc in track(dl_mapping, 'download'):
         dl_base = '_'.join([acc, dl_mapping[acc]['assem_name']])
         dl_base = dl_base.replace(',', '')
         for ft in o:
@@ -911,7 +926,7 @@ def download_genomes(o, dl_mapping):
                 # dl = dl_gz.replace('.gz', '')
                 # os.rename(dl, out)
                 if os.path.exists(out):
-                    sys.stderr.write('\n')
+                    # sys.stderr.write('\n')
                     logger.info(
                         f'{out} successfully downloaded.'
                     )
